#!/bin/python

'''
@author: Marius Lindauer
@license: GPLv3
'''

import os,sys, shutil
from subprocess import Popen

############################################
## DON'T USE "_" in scenario and algo name
#############################################

scenarios = {
              #INDUSTRIAL
             "CSSC-IBM-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-IBM-300s-2day.txt",
                   "track": "industrial"
                  },
            "CSSC-BMC08-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-BMC08-300s-2day.txt",
                  "track": "industrial"
                 },
            #===================================================================
            # "CSSC-SWV-300s-2days" :
            #     {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-SWV-300s-2days.txt",
            #       "track": "industrial"
            #      },
            #===================================================================
              "CSSC-CircuitFuzz-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-CircuitFuzz-300s-2day.txt",
                   "track": "industrial"
                  },
            # CRAFTED
             "CSSC-LABS-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-LABS-300s-2day.txt",
                   "track": "crafted"
                  },
             "CSSC-GI-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-GI-300s-2day.txt",
                   "track": "crafted"
                  },
            "CSSC-Queens-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-NQUEENS-300s-2day.txt",
                  "track": "crafted"
                 },
          
             #RANDOM
             "CSSC-3cnf-v350-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-3cnf_v350-300s-2day.txt",
                   "track": "random"
                  },     
              "CSSC-unsat-unif-k5-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-unsat-unif-k5-300s-2day.txt",
                   "track": "random"
                  },  
               "CSSC-K3-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-K3-300s-2day.txt",
                   "track": "random"
                  },
              # RANDOMSAT
              "CSSC-3SAT1k-sat-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-3SAT1k-sat-300s-2day.txt",
                   "track": "randomSAT"
                  },
             "CSSC-5SAT500-sat-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-5SAT500-sat-300s-2day.txt",
                   "track": "randomSAT"
                  },
             "CSSC-7SAT90-sat-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-7SAT90-sat-300s-2day.txt",
                   "track": "randomSAT"
                  },

             }

#------------------------ algos = {"minisat-HACK-999ED-CSSC" : { "ruby" : False,
                             # "wrapper" : "./solvers/minisat_HACK_999ED_CSSC/minisat_HACK_999ED_CSSCWrapper.py",
                             # "pcs": "./solvers/minisat_HACK_999ED_CSSC/minisat_HACK_999ED_CSSC-params.pcs",
                             # "tracks" : ["industrial", "crafted", "random", "randomSAT"]
                       #------------------------------------------------------ }
         #-------------------------------------------------------------------- }
 
 
algos = {"SparrowToRiss" : { "ruby" : False,
                             "wrapper" : "./solvers/SparrowToRiss/StrWrapper.py",
                             "pcs": "./solvers/SparrowToRiss/SparrowToRiss-CSSC.pcs",
                             "tracks" : ["industrial", "crafted", "random", "randomSAT"]
                            },
    	"probSAT" : { "ruby" : False,
                                 "wrapper" : "./solvers/probSAT/probSATWrapper.py",
                                 "pcs": "./solvers/probSAT/probSAT_params.pcs",
                                 "tracks" : ["randomSAT"] #TODO: Remove industrial
                                },
         "Riss-4.27" : { "ruby" : False,
                             "wrapper" : "./solvers/riss427/rissWrapper.py",
                             "pcs": "./solvers/riss427/riss427-CSSC.pcs",
                             "tracks" : ["industrial", "crafted", "random"]
                            },
         "cryptominisat" : { "ruby" : True,
                             "wrapper" : "./solvers/cryptominisat/callstring_generator.rb",
                             "pcs": "./solvers/cryptominisat/cryptominisat-params.pcs",
                             "tracks" : ["industrial", "crafted"]
                       },
         "minisat-HACK-999ED-CSSC" : { "ruby" : False,
                             "wrapper" : "./solvers/minisat_HACK_999ED_CSSC/minisat_HACK_999ED_CSSCWrapper.py",
                             "pcs": "./solvers/minisat_HACK_999ED_CSSC/minisat_HACK_999ED_CSSC-params.pcs",
                             "tracks" : ["industrial", "crafted", "random", "randomSAT"]
                       },
         "clasp-3.0.4-p8" : { "ruby" : False,
                     "wrapper" : "./solvers/clasp-cssc/claspCSSCWrapper.py",
                     "pcs": "./solvers/clasp-cssc/clasp-sat-params-nat.pcs",
                     "pcs-discrete": "./solvers/clasp-cssc/clasp-sat-params-cat.pcs",
                     "tracks" : ["industrial", "crafted", "random", "randomSAT"]
                      },
         "DCCASat+march-rw" : { "ruby" : True,
                             "wrapper" : "./solvers/DCCASat+march_rw/callstring_generator.rb",
                             "pcs": "solvers/DCCASat+march_rw/DCCASat+march_rw-params.pcs",
                             "tracks" : ["random"]
                            },
         "CSCCSat2014" : { "ruby" : True,
                           "wrapper" : "./solvers/CSCCSat2014/callstring_generator.rb",
                           "pcs": "solvers/CSCCSat2014/CSCCSat2014-params.pcs",
                            "tracks" : ["randomSAT"]
                        },
        "lingeling" : { "ruby" : True,
                        "wrapper" : "./solvers/lingeling/callstring_generator.rb",
                        "pcs": "solvers/lingeling/lingeling-params.pcs",
                        "pcs-discrete": "./solvers/lingeling/lingeling-params-mixed.pcs",
                        "tracks" : ["industrial", "crafted"]
                        },
         "YalSAT" : { "ruby" : True,
                "wrapper" : "./solvers/yalsat/callstring_generator.rb",
                "pcs": "solvers/yalsat/yalsat-params.pcs",
                "pcs-discrete": "solvers/yalsat/yalsat-params-mixed.pcs",
                "tracks" : ["crafted", "randomSAT"]
                }
         } 

cwd = os.getcwd()

cssc_folder = "/data/aad/cssc14/cssc/"
configurators = ["smac", "paramils", "gga", "smac-disc", "gga-disc"]
#configurators = ["smac-disc", "gga-disc"]

REPETITIONS = 4
WC_BUDGET =  172800
BUILD_PCS = True
SMAC_PATH = "./configurators/smac-v2.06.01-master-622/"

for configurator in configurators:
    for algo_name, algo in algos.items():
        for scenario_name, scenario in scenarios.items():
            # verify that scenario's track and algo
            if scenario["track"] not in algo["tracks"]:
                continue

            id = "%s_%s_%s_%s" %(scenario["track"], scenario_name, configurator, algo_name)

            os.chdir(cwd)
            new_dir_name = id 
            if not os.path.isdir(new_dir_name):
                os.mkdir(new_dir_name)
            os.chdir(new_dir_name)
            print(os.getcwd())
            
            if not os.path.islink("./instances"):
                os.symlink(os.path.join(cssc_folder,"instances"), "./instances")
                os.symlink(os.path.join(cssc_folder,"scripts"), "./scripts")
                os.symlink(os.path.join(cssc_folder,"solvers"), "./solvers")
                os.symlink(os.path.join(cssc_folder,"configurators"), "./configurators")
                os.symlink(os.path.join(cssc_folder,"scenarios"), "./scenarios")

            if not os.path.isfile(scenario["file"]):
                sys.stderr.write("[ERROR]: Have not found: %s\n" %(scenario["file"]))
                continue
            if not os.path.isfile(algo["pcs"]):
                sys.stderr.write("[ERROR]: Have not found: %s\n" %(algo["pcs"]))
                continue
            if not os.path.isfile(algo["wrapper"]):
                sys.stderr.write("[ERROR]: Have not found: %s\n" %(algo["wrapper"]))
                continue
            
            fp_new = open("./%s.txt" %(scenario_name), "w")
            fp_old = open(scenario["file"], "r")
            
            inst_file = None
            
            for line in fp_old:
                if configurator == "gga" and line.startswith("tunerTimeout"):
                    fp_new.write("tunerTimeout = 2147483647\n") # maxint 32bit
                else:
                    fp_new.write(line)
                if line.startswith("instance_file"):
                    inst_file = line.strip("\n").split("=")[1].strip(" ")
                
            with open(inst_file) as fp:
                n_train_inst = len(fp.readlines()) - 1
            
              
            
            if algo["ruby"]:
                if configurator in ["gga", "gga-disc"]:
                    fp_bash = open("run-algo.sh", "w")
                    fp_bash.write("#!\\bin\\bash\n")
                    fp_bash.write("python -u scripts/SATCSSCWrapper.py --mem-limit 3072 --script None --ext-callstring \"ruby %s\" --sat-checker ./scripts/SAT --sol-file ./instances/true_solubility.txt --log True $*" %(algo["wrapper"]))
                    fp_bash.close()
                    fp_new.write("algo = bash run-algo.sh \n" )
                else:
                    fp_new.write("algo = python -u scripts/SATCSSCWrapper.py --mem-limit 3072 --script None --ext-callstring \"ruby %s\" --sat-checker ./scripts/SAT --sol-file ./instances/true_solubility.txt --log True \n" %(algo["wrapper"]))
            else:
                fp_new.write("algo = python -u scripts/SATCSSCWrapper.py --mem-limit 3072 --script %s --sat-checker ./scripts/SAT --sol-file ./instances/true_solubility.txt --log True \n" %(algo["wrapper"]) )
            
            fp_old.close()

            fp_run = open("run.sh", "w")
            fp_run.write("#!/bin/bash\n")
            
            # tmp directory for Riss and SparrowToRiss
            fp_run.write("mkdir tmp\n")
            if configurator == "gga":
                
                # generate gga-style pcs file
                if BUILD_PCS:
                    pcs_xml = open(algo["pcs"]+".xml", "w")
                    cmd  = ["python","configurators/gga-1.3.2/paramils_convert.py", algo["pcs"]]
                    print(" ".join(cmd))
                    io = Popen(cmd, stdout=pcs_xml)
                    io.communicate()
                    pcs_xml.close()
                fp_new.write("paramfile = %s\n" %(algo["pcs"]))
                fp_new.write("wallClockLimit = %d\n" %(WC_BUDGET))
                #generate run script for GGA
                fp_run.write("./configurators/gga-1.3.2/gga.sh /dev/null /dev/null -scen ./%s -p 50 -g 100 -gf 75 -is 4 -ie %d -v 5 -traj ./traj.csv -t 4 1> ./log.txt 2>&1\n" %(fp_new.name, n_train_inst))
                fp_run.write(SMAC_PATH + "smac-validate --trajectoryFile ./traj.csv --numRun 1 --outputFileSuffix csscWorkerValid_mult2 --numValidationRuns 1 --maxTimestamp 172800 --minTimestamp 0.01 --multFactor 2 --tae CLI --scenarioFile %s --validateOnlyLastIncumbent true --useScenarioOutDir true --outputDirectory ./validate-train --validateTestInstances false --waitForPersistedRunCompletion false 1> ./log-validate-train.txt 2>&1\n"  %(fp_new.name))
                fp_run.write(SMAC_PATH + "smac-validate --trajectoryFile ./traj.csv --numRun 1 --outputFileSuffix csscWorkerValid_mult2 --numValidationRuns 1 --maxTimestamp 172800 --minTimestamp 0.01 --multFactor 2 --tae CLI --scenarioFile %s --validateOnlyLastIncumbent true --useScenarioOutDir true --outputDirectory ./validate-test --validateTestInstances true --waitForPersistedRunCompletion false 1> ./log-validate-test.txt 2>&1\n"  %(fp_new.name))
            elif configurator == "paramils":
                # generate discretized pcs file for ParamILS
                if BUILD_PCS:
                    pcs_disc = open(algo["pcs"]+".discretized.pcs", "w")
                    cmd = ["python", "scripts/discretize.py", algo["pcs"]]
                    print(" ".join(cmd))
                    io = Popen(cmd, stdout=pcs_disc)
                    io.communicate()
                    pcs_disc.close()
                if algo.get("pcs-discrete"):
                    fp_new.write("paramfile = %s\n" %(algo["pcs-discrete"]))
                else:
                    fp_new.write("paramfile = %s.discretized.pcs\n" %(algo["pcs"]))
                #ParamILS does not support this option : -(
                fp_new.write("wallclock-limit = %d\n" %(WC_BUDGET))
                #generate run script for ParamILS
                fp_run.write("./configurators/paramils/param_ils_2_3_run.rb -init 1 -pruning 1 -N 2000 -validN 0 -userunlog 0 -numRun $1 -scenariofile %s 1> ./log$1.txt 2>&1\n" %(fp_new.name))
                fp_run.write(SMAC_PATH + "smac-validate --trajectoryFile paramils_out/focused-runs2000-runobjruntime-overallobjmean10-maxit200000000-bm2-time300.0-tunerTime172800.0-algoAlgo-traj_$1.csv --numRun $1 --outputFileSuffix csscWorkerValid_mult2 --numValidationRuns 1 --maxTimestamp 172800 --minTimestamp 0.01 --multFactor 2 --tae CLI --scenarioFile %s --validateOnlyLastIncumbent true --useScenarioOutDir true --outputDirectory ./validate-train --validateTestInstances false --waitForPersistedRunCompletion false 1> ./log-validate-train-$1.txt 2>&1\n"  %(fp_new.name))
                fp_run.write(SMAC_PATH + "smac-validate --trajectoryFile paramils_out/focused-runs2000-runobjruntime-overallobjmean10-maxit200000000-bm2-time300.0-tunerTime172800.0-algoAlgo-traj_$1.csv --numRun $1 --outputFileSuffix csscWorkerValid_mult2 --numValidationRuns 1 --maxTimestamp 172800 --minTimestamp 0.01 --multFactor 2 --tae CLI --scenarioFile %s --validateOnlyLastIncumbent true --useScenarioOutDir true --outputDirectory ./validate-test --validateTestInstances true --waitForPersistedRunCompletion false 1> ./log-validate-test-$1.txt 2>&1\n"  %(fp_new.name))
                #TODO: use above line
                #fp_run.write(SMAC_PATH + "smac-validate --trajectoryFile paramils_out/focused-runs2000-runobjruntime-overallobjmean10-maxit200000000-bm2-time5.0-tunerTime3600.0-algoAlgo-traj_$1.csv --numRun $1 --outputFileSuffix csscWorkerValid_mult2 --numValidationRuns 1 --maxTimestamp 172800 --minTimestamp 0.01 --multFactor 2 --tae CLI --scenarioFile %s --validateOnlyLastIncumbent true --useScenarioOutDir true --outputDirectory ./validate-train  --validateTestInstances false --waitForPersistedRunCompletion false 1> ./log-validate-train-$1.txt 2>&1\n"  %(fp_new.name))
                #fp_run.write(SMAC_PATH + "smac-validate --trajectoryFile paramils_out/focused-runs2000-runobjruntime-overallobjmean10-maxit200000000-bm2-time5.0-tunerTime3600.0-algoAlgo-traj_$1.csv --numRun $1 --outputFileSuffix csscWorkerValid_mult2 --numValidationRuns 1 --maxTimestamp 172800 --minTimestamp 0.01 --multFactor 2 --tae CLI --scenarioFile %s --validateOnlyLastIncumbent true --useScenarioOutDir true --outputDirectory ./validate-test --validateTestInstances true --waitForPersistedRunCompletion false 1> ./log-validate-test-$1.txt 2>&1\n"  %(fp_new.name))
            elif configurator == "smac":
                fp_new.write("paramfile = %s\n" %(algo["pcs"]))
                fp_new.write("wallClockLimit = %d\n" %(WC_BUDGET))
                #generate run script for SMAC
                fp_run.write(SMAC_PATH + "smac --numRun $1 --doValidation False --scenarioFile %s --rungroup CSSC 1> ./log$1.txt 2>&1\n" %(fp_new.name))
                fp_run.write(SMAC_PATH + "smac-validate --trajectoryFile smac-output/CSSC/traj-run-$1.csv --numRun $1 --outputFileSuffix csscWorkerValid_mult2 --numValidationRuns 1 --maxTimestamp 172800 --minTimestamp 0.01 --multFactor 2 --tae CLI --scenarioFile %s --validateOnlyLastIncumbent true --useScenarioOutDir true --outputDirectory ./validate-train --validateTestInstances false --waitForPersistedRunCompletion false 1> ./log-validate-train-$1.txt 2>&1\n"  %(fp_new.name))
                fp_run.write(SMAC_PATH + "smac-validate --trajectoryFile smac-output/CSSC/traj-run-$1.csv --numRun $1 --outputFileSuffix csscWorkerValid_mult2 --numValidationRuns 1 --maxTimestamp 172800 --minTimestamp 0.01 --multFactor 2 --tae CLI --scenarioFile %s --validateOnlyLastIncumbent true --useScenarioOutDir true --outputDirectory ./validate-test --validateTestInstances true --waitForPersistedRunCompletion false 1> ./log-validate-test-$1.txt 2>&1\n"  %(fp_new.name))
            
            elif configurator == "smac-disc":
                # generate discretized pcs file for ParamILS
                if BUILD_PCS:
                    pcs_disc = open(algo["pcs"]+".discretized.pcs", "w")
                    cmd = ["python", "scripts/discretize.py", algo["pcs"]]
                    print(" ".join(cmd))
                    io = Popen(cmd, stdout=pcs_disc)
                    io.communicate()
                    pcs_disc.close()
                if algo.get("pcs-discrete"):
                    fp_new.write("paramfile = %s\n" %(algo["pcs-discrete"]))
                else:
                    fp_new.write("paramfile = %s.discretized.pcs\n" %(algo["pcs"]))
                
                fp_new.write("wallClockLimit = %d\n" %(WC_BUDGET))
                #generate run script for SMAC
                fp_run.write(SMAC_PATH + "smac --numRun $1 --doValidation False --scenarioFile %s --rungroup CSSC 1> ./log$1.txt 2>&1\n" %(fp_new.name))
                fp_run.write(SMAC_PATH + "smac-validate --trajectoryFile smac-output/CSSC/traj-run-$1.csv --numRun $1 --outputFileSuffix csscWorkerValid_mult2 --numValidationRuns 1 --maxTimestamp 172800 --minTimestamp 0.01 --multFactor 2 --tae CLI --scenarioFile %s --validateOnlyLastIncumbent true --useScenarioOutDir true --outputDirectory ./validate-train --validateTestInstances false --waitForPersistedRunCompletion false 1> ./log-validate-train-$1.txt 2>&1\n"  %(fp_new.name))
                fp_run.write(SMAC_PATH + "smac-validate --trajectoryFile smac-output/CSSC/traj-run-$1.csv --numRun $1 --outputFileSuffix csscWorkerValid_mult2 --numValidationRuns 1 --maxTimestamp 172800 --minTimestamp 0.01 --multFactor 2 --tae CLI --scenarioFile %s --validateOnlyLastIncumbent true --useScenarioOutDir true --outputDirectory ./validate-test --validateTestInstances true --waitForPersistedRunCompletion false 1> ./log-validate-test-$1.txt 2>&1\n"  %(fp_new.name))
            elif configurator == "gga-disc":
                # generate gga-style pcs file
                if BUILD_PCS:
                    # first build discretized pcs
                    pcs_disc = open(algo["pcs"]+".discretized.pcs", "w")
                    cmd = ["python", "scripts/discretize.py", algo["pcs"]]
                    print(" ".join(cmd))
                    io = Popen(cmd, stdout=pcs_disc)
                    io.communicate()
                    pcs_disc.close()
                    # generate xml pcs in gga format
                    pcs_xml = open(algo["pcs"]+".discretized.pcs.xml", "w")
                    cmd  = ["python","configurators/gga-1.3.2/paramils_convert.py", algo["pcs"]+".discretized.pcs"]
                    print(" ".join(cmd))
                    io = Popen(cmd, stdout=pcs_xml)
                    io.communicate()
                    pcs_xml.close()
                fp_new.write("paramfile = %s.discretized.pcs\n" %(algo["pcs"]))
                fp_new.write("wallClockLimit = %d\n" %(WC_BUDGET))
                #generate run script for GGA
                fp_run.write("./configurators/gga-1.3.2/gga.sh /dev/null /dev/null -scen ./%s -p 50 -g 100 -gf 75 -is 4 -ie %d -v 5 -traj ./traj.csv -t 4 1> ./log.txt 2>&1\n" %(fp_new.name, n_train_inst))
                fp_run.write(SMAC_PATH + "smac-validate --trajectoryFile ./traj.csv --numRun 1 --outputFileSuffix csscWorkerValid_mult2 --numValidationRuns 1 --maxTimestamp 172800 --minTimestamp 0.01 --multFactor 2 --tae CLI --scenarioFile %s --validateOnlyLastIncumbent true --useScenarioOutDir true --outputDirectory ./validate-train --validateTestInstances false --waitForPersistedRunCompletion false 1> ./log-validate-train.txt 2>&1\n"  %(fp_new.name))
                fp_run.write(SMAC_PATH + "smac-validate --trajectoryFile ./traj.csv --numRun 1 --outputFileSuffix csscWorkerValid_mult2 --numValidationRuns 1 --maxTimestamp 172800 --minTimestamp 0.01 --multFactor 2 --tae CLI --scenarioFile %s --validateOnlyLastIncumbent true --useScenarioOutDir true --outputDirectory ./validate-test --validateTestInstances true --waitForPersistedRunCompletion false 1> ./log-validate-test.txt 2>&1\n"  %(fp_new.name))
            
            fp_job = open("%s.sge" %(id), "w")
            if configurator in ["smac", "paramils", "smac-disc"]:
                for i in range(0, REPETITIONS):
                    fp_job.write("bash ./run.sh %d\n" %(i))
            else:
                fp_job.write("bash ./run.sh\n")
                
            if configurator in ["smac"]:
                fp_job.write(SMAC_PATH + "smac-validate --numRun 1 --numValidationRuns 1 --tae CLI --scenarioFile %s --useScenarioOutDir true --outputDirectory ./validate-default --validateTestInstances true --waitForPersistedRunCompletion false --includeDefaultAsFirstRandom true --random-configurations 1 1> log-default.txt 2>&1\n" %(fp_new.name))
                
            #fp_run.write("rm -rf tmp-$1\n")
            fp_new.close()
            fp_run.close()
            fp_job.close()
