#!/bin/python

'''
@author: Marius Lindauer
@license: GPLv3
'''

import os,sys, shutil
from subprocess import Popen

############################################
## DON'T USE "_" in scenario and algo name
#############################################

scenarios = {
              #INDUSTRIAL
             "CSSC-IBM-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-IBM-300s-2day.txt",
                   "track": "industrial"
                  },
            "CSSC-BMC08-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-BMC08-300s-2day.txt",
                  "track": "industrial"
                 },
            #===================================================================
            # "CSSC-SWV-300s-2days" :
            #     {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-SWV-300s-2days.txt",
            #       "track": "industrial"
            #      },
            #===================================================================
              "CSSC-CircuitFuzz-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-CircuitFuzz-300s-2day.txt",
                   "track": "industrial"
                  },
            # CRAFTED
             "CSSC-LABS-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-LABS-300s-2day.txt",
                   "track": "crafted"
                  },
             "CSSC-GI-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-GI-300s-2day.txt",
                   "track": "crafted"
                  },
            "CSSC-Queens-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-NQUEENS-300s-2day.txt",
                  "track": "crafted"
                 },
          
             #RANDOM
             "CSSC-3cnf-v350-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-3cnf_v350-300s-2day.txt",
                   "track": "random"
                  },     
              "CSSC-unsat-unif-k5-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-unsat-unif-k5-300s-2day.txt",
                   "track": "random"
                  },  
               "CSSC-K3-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-K3-300s-2day.txt",
                   "track": "random"
                  },
              # RANDOMSAT
              "CSSC-3SAT1k-sat-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-3SAT1k-sat-300s-2day.txt",
                   "track": "randomSAT"
                  },
             "CSSC-5SAT500-sat-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-5SAT500-sat-300s-2day.txt",
                   "track": "randomSAT"
                  },
             "CSSC-7SAT90-sat-300s-2day" :
                 {"file" :  "/data/aad/cssc14/cssc/scenarios/CSSC-7SAT90-sat-300s-2day.txt",
                   "track": "randomSAT"
                  },

             }

#------------------------ algos = {"minisat-HACK-999ED-CSSC" : { "ruby" : False,
                             # "wrapper" : "./solvers/minisat_HACK_999ED_CSSC/minisat_HACK_999ED_CSSCWrapper.py",
                             # "pcs": "./solvers/minisat_HACK_999ED_CSSC/minisat_HACK_999ED_CSSC-params.pcs",
                             # "tracks" : ["industrial", "crafted", "random", "randomSAT"]
                       #------------------------------------------------------ }
         #-------------------------------------------------------------------- }
 
 
algos = {"SparrowToRiss" : { "ruby" : False,
                             "wrapper" : "./solvers/SparrowToRiss/StrWrapper.py",
                             "pcs": "./solvers/SparrowToRiss/SparrowToRiss-CSSC.pcs",
                             "tracks" : ["industrial", "crafted", "random", "randomSAT"]
                            },
    	"probSAT" : { "ruby" : False,
                                 "wrapper" : "./solvers/probSAT/probSATWrapper.py",
                                 "pcs": "./solvers/probSAT/probSAT_params.pcs",
                                 "tracks" : ["randomSAT"] #TODO: Remove industrial
                                },
         "Riss-4.27" : { "ruby" : False,
                             "wrapper" : "./solvers/riss427/rissWrapper.py",
                             "pcs": "./solvers/riss427/riss427-CSSC.pcs",
                             "tracks" : ["industrial", "crafted", "random"]
                            },
         "cryptominisat" : { "ruby" : True,
                             "wrapper" : "./solvers/cryptominisat/callstring_generator.rb",
                             "pcs": "./solvers/cryptominisat/cryptominisat-params.pcs",
                             "tracks" : ["industrial", "crafted"]
                       },
         "minisat-HACK-999ED-CSSC" : { "ruby" : False,
                             "wrapper" : "./solvers/minisat_HACK_999ED_CSSC/minisat_HACK_999ED_CSSCWrapper.py",
                             "pcs": "./solvers/minisat_HACK_999ED_CSSC/minisat_HACK_999ED_CSSC-params.pcs",
                             "tracks" : ["industrial", "crafted", "random", "randomSAT"]
                       },
         "clasp-3.0.4-p8" : { "ruby" : False,
                     "wrapper" : "./solvers/clasp-cssc/claspCSSCWrapper.py",
                     "pcs": "./solvers/clasp-cssc/clasp-sat-params-nat.pcs",
                     "pcs-discrete": "./solvers/clasp-cssc/clasp-sat-params-cat.pcs",
                     "tracks" : ["industrial", "crafted", "random", "randomSAT"]
                      },
         "DCCASat+march-rw" : { "ruby" : True,
                             "wrapper" : "./solvers/DCCASat+march_rw/callstring_generator.rb",
                             "pcs": "solvers/DCCASat+march_rw/DCCASat+march_rw-params.pcs",
                             "tracks" : ["random"]
                            },
         "CSCCSat2014" : { "ruby" : True,
                           "wrapper" : "./solvers/CSCCSat2014/callstring_generator.rb",
                           "pcs": "solvers/CSCCSat2014/CSCCSat2014-params.pcs",
                            "tracks" : ["randomSAT"]
                        },
        "lingeling" : { "ruby" : True,
                        "wrapper" : "./solvers/lingeling/callstring_generator.rb",
                        "pcs": "solvers/lingeling/lingeling-params.pcs",
                        "pcs-discrete": "./solvers/lingeling/lingeling-params-mixed.pcs",
                        "tracks" : ["industrial", "crafted"]
                        },
         "YalSAT" : { "ruby" : True,
                "wrapper" : "./solvers/yalsat/callstring_generator.rb",
                "pcs": "solvers/yalsat/yalsat-params.pcs",
                "pcs-discrete": "solvers/yalsat/yalsat-params-mixed.pcs",
                "tracks" : ["crafted", "randomSAT"]
                }
         } 

cwd = os.getcwd()

cssc_folder = "/data/aad/cssc14/cssc/"
configurators = ["smac"] # once in smac should be enough

SMAC_PATH = "./configurators/smac-v2.06.01-master-622/"
grid_help = "/home/lindauer/git/metahelper/grid_helper.py"

for configurator in configurators:
    for algo_name, algo in algos.items():
        for scenario_name, scenario in scenarios.items():
            # verify that scenario's track and algo
            if scenario["track"] not in algo["tracks"]:
                continue

            id = "%s_%s_%s_%s" %(scenario["track"], scenario_name, configurator, algo_name)

            os.chdir(cwd)
            new_dir_name = id 
            os.chdir(new_dir_name)
            print(os.getcwd())
            
            if not os.path.isfile(scenario["file"]):
                sys.stderr.write("[ERROR]: Have not found: %s\n" %(scenario["file"]))
                continue
            if not os.path.isfile(algo["pcs"]):
                sys.stderr.write("[ERROR]: Have not found: %s\n" %(algo["pcs"]))
                continue
            if not os.path.isfile(algo["wrapper"]):
                sys.stderr.write("[ERROR]: Have not found: %s\n" %(algo["wrapper"]))
                continue
            
            fp_job = open("%s-train-def.sge" %(id), "w")
            fp_job.write(SMAC_PATH + "smac-validate --numRun 1 --numValidationRuns 1 --tae CLI --scenarioFile %s.txt --useScenarioOutDir true --outputDirectory ./validate-default-train --validateTestInstances false --waitForPersistedRunCompletion false --includeDefaultAsFirstRandom true --random-configurations 1 1> log-default-train.txt 2>&1\n" %(scenario_name))
            fp_job.close()
            
            cmd = ["python" , grid_help, fp_job.name, "-q", "aad_core.q", "--lr", "-l", "."]
            print(" ".join(cmd))
            p = Popen(cmd)
            p.communicate()
            
            