#!/bin/python

'''
@author: Marius Lindauer
@license: GPLv3
'''

import os, sys
import re
import math

def estimate_pcs(pcs_file):
    ''' '''
    CONT_REGEX = re.compile("^[ ]*(?P<name>[^ ]+)[ ]*\[(?P<range_start>-?[0-9]+(\.[0-9]+)?)[ ]*,[ ]*(?P<range_end>-?[0-9]+(\.[0-9]+)?)\][ ]*\[(?P<default>[^#]*)\](?P<misc>.*)$")
    CAT_REGEX = re.compile("^[ ]*(?P<name>[^ ]+)[ ]*{(?P<values>.+)}[ ]*\[(?P<default>[^#]*)\](?P<misc>.*)$")
    COND_REGEX = re.compile("^[ ]*(?P<cond>[^ ]+)[ ]*\|[ ]*(?P<head>[^ ]+)[ ]*in[ ]*{(?P<values>.+)}(?P<misc>.*)$")
        
    upper_bound = 1 # consider continuous and categorical parameters
    lower_bound = 1 # consider only categorical parameters
    params_in_conditions = set()
    parameters = {"c": 0, "i": 0, "r": 0}
    with open(pcs_file) as fp:
        for line in fp:
            line = line.strip("\n")
            
            if line.find("#") > -1:
                    line = line[:line.find("#")] # remove comments
                    
            cont_match = CONT_REGEX.match(line)
            cat_match = CAT_REGEX.match(line)
            cond_match = COND_REGEX.match(line)
            if cont_match:
                if "i" in cont_match.group("misc"):
                    upper_bound *= 1 + int(cont_match.group("range_end")) - int(cont_match.group("range_start"))
                    parameters["i"] += 1
                else:
                    upper_bound = float("inf")
                    parameters["r"] += 1 
                
            elif cat_match:
                n_values = len(cat_match.group("values").split(","))
                lower_bound *= n_values
                upper_bound *= n_values
                parameters["c"] += 1
            elif cond_match:
                params_in_conditions.add(cond_match.group("cond"))
            elif line:
                #sys.stderr.write("Not parsed: %s\n" %(line))
                pass
                
    conditions = len(params_in_conditions)
    try:
        print("%-80s : \t p: %-10s cond: %-3d %.1g - %1.g" %(pcs_file, str(parameters), conditions, lower_bound, upper_bound))
    except:
        print("%-80s : \t p: %-10s cond: %-3d %.1g - e+%0.f" %(pcs_file, str(parameters), conditions, lower_bound, int(math.log10(upper_bound))))
        

# sys.argv[1] : folder with target algorithms

if os.path.isdir(sys.argv[1]):
    for root, dirs, files in os.walk(sys.argv[1], followlinks=False):
        for file_ in files:
            if file_.endswith(".pcs"):
               estimate_pcs(os.path.join(root,file_)) 
elif os.path.isfile(sys.argv[1]):
    estimate_pcs(sys.argv[1])
else:
    sys.stderr.write("Could not read input\n")
    
