'''
@author: Marius Lindauer
@license: GPLv3
'''

import os
import sys
import re
import logging

class READ_AND_WRITE_CSV(object):
    '''
         gets as input cssc log directory, reads input data and writes them as csv files
    '''
    
    def __init__(self):
        ''' Constructor '''
        
    def main(self, root):
        '''
            main method
            Arguments:
                root: root directory for log files
        '''
        
        main_dict = self.read_data_14(root)
        if not main_dict:
            main_dict = self.read_data_13(root)
            
        if not main_dict:
            logging.error("Found no data")
            sys.exit(3)
            
        self.write_csv(main_dict)
        
    
    def read_csv(self, fp, cutoff=300):
        '''
            reads validationInstanceSeedResult-tunertime-run%d.csv
            (Format: "Configuration","Seed","Instance","Response")
            and returns PAR1, PAR10, #TOs and dict: inst_name -> performance
            
        '''
        par1 = 0.0
        par10 = 0.0
        tos = 0
        insts = 0
        inst_perf_dict = {}
        for line in fp:
            _,_,instance,response = line.split(",")
            try:
                runtime = float(response.strip("\n").strip("\""))
            except:
                continue
            instance = instance.strip("\"")
            inst_perf_dict[instance] = runtime
            par1 += runtime
            if runtime >= cutoff:
                runtime *= 10
                tos += 1
            par10 += runtime
            insts +=1
        return par1/insts, par10/insts, tos, inst_perf_dict
    

    def read_data_14(self, root, REPETITIONS=4):
        '''
            Reads data from 2014
            Returns main_dict with 
                track -> 
                    scenario -> 
                        solver_dict -> 
                            configurator/default/default-train ->  
                                "training_perf"/"test_perf" -> 
                                    "PAR1"/"PAR10"/"TOs"/"Performance" 
        '''
        logging.info("Try to read CSSC14 data")
        main_dict = {} 
        for dir_ in os.listdir(root):
            dir_full = os.path.join(root, dir_)
            if not os.path.isdir(dir_full):
                continue
            try:
                track, scenario, configurator, solver = dir_.split("_")
            except:
                continue
            dir_ = dir_full
            logging.info(dir_) 
            main_dict[track] = main_dict.get(track, {})
            track_dict = main_dict[track]
            track_dict[scenario] = track_dict.get(scenario, {})
            scenario_dict = track_dict[scenario]
            scenario_dict[solver] = scenario_dict.get(solver, {})
            solver_dict = scenario_dict[solver]
            
            train_val_dir = os.path.join(dir_,"validate-train")
            test_val_dir = os.path.join(dir_,"validate-test")  
            if configurator == "gga":
                runs = [1]
            else:
                runs = range(0, REPETITIONS)
                
            for r in runs:
                # TRAINING PERFORMANCE
                csv_results = os.path.join(train_val_dir, "validationInstanceSeedResult-tunertimecsscWorkerValid_mult2-run%d.csv" %(r))
                if not os.path.isfile(csv_results) and (r == 1 or "gga" not in configurator) :
                    sys.stderr.write("Missing: %s\n" %(csv_results))
                if not os.path.isfile(csv_results):
                    continue
                
                fp = open(csv_results, "r")
                par1, par10, tos, perf_dict = self.read_csv(fp)
                training_performance  = {"PAR1": par1, "PAR10": par10, "TOs": tos, "Performance": perf_dict}
                fp.close()
                
                #TEST PERFORMANCE
                csv_results = os.path.join(test_val_dir, "validationInstanceSeedResult-tunertimecsscWorkerValid_mult2-run%d.csv" %(r))
                if not os.path.isfile(csv_results):
                    sys.stderr.write("Missing: %s\n" %(csv_results))
                    continue
                fp = open(csv_results, "r")
                par1, par10, tos, perf_dict = self.read_csv(fp)
                test_performance  = {"PAR1": par1, "PAR10": par10, "TOs": tos, "Performance": perf_dict}
                #print(tos)
                
                solver_dict[configurator+"-"+str(r)] = {"training_perf" : training_performance, "test_perf": test_performance}
                fp.close()
                
            if configurator == "smac":
                default_file = os.path.join(dir_,"validate-default","validationInstanceSeedResult-tunertime-run1.csv")
                if not os.path.isfile(default_file):
                    logging.warn("Missing: %s" %(default_file))
                else:
                    fp = open(default_file, "r")
                    par1, par10, tos, perf_dict = self.read_csv(fp)
                    solver_dict["default"] = {"PAR1": par1, "PAR10": par10, "TOs": tos, "Performance": perf_dict}
                    
                default_file = os.path.join(dir_,"validate-default-train","validationInstanceSeedResult-tunertime-run1.csv")
                if not os.path.isfile(default_file):
                    sys.stderr.write("Missing: %s\n" %(default_file))
                else:
                    fp = open(default_file, "r")
                    par1, par10, tos, perf_dict = self.read_csv(fp)
                    solver_dict["default-train"] = {"PAR1": par1, "PAR10": par10, "TOs": tos, "Performance": perf_dict}
                
        return main_dict

    def read_data_13(self, root):
        '''
            Reads data from 2013
            Returns main_dict with 
                track -> 
                    scenario -> 
                        solver_dict -> 
                            configurator/default/default-train ->  
                                "training_perf"/"test_perf" -> 
                                    "PAR1"/"PAR10"/"TOs"/"Performance" 
        '''
        
        logging.info("Try to read CSSC13 data")
        
        configurator_dic  = {"pils": "paramils", "smac":"smac"}
        
        mapping_scenario_track ={ "CSSC-IBM-300s-2day": "industrial",
                                  "CSSC-5SAT500-sat-300s-2day" : "random",
                                  "CSSC-CircuitFuzz-300s-2day" : "industrial",
                                  "CSSC-GI-300s-2day": "crafted",
                                  "CSSC-BMC08-300s-2day" : "industrial",
                                  "CSSC-LABS-300s-2day" : "crafted",
                                  "CSSC-K3-300s-2day": "random",
                                  "CSSC-unsat-unif-k5-300s-2day": "random",
                                  "CSSC-SWV-300s-2days" : "industrial"
                                 }
        
        mapping_solver_name = {"riss3gExt" : "rissgExt",
                               "riss3g" : "rissg",
                               "Solver43": "SolverFourtyThree",
                               "forl-nodrup" : "forlnodrup",
                               "sat4j" : "satj",
                               "gnovelty+GCwa" : "gnoveltyGCwa",
                               "gnovelty+PCL" : "gnoveltyPCL",
                               "gnovelty+GCa" : "gnoveltyGCa"
                               }
        
        main_dict = {}
        config_test_regex = re.compile("validationInstanceSeedResult-csscWorkerValidFinalIncTest-run(\d)\.csv")
        config_train_regex = re.compile("validationInstanceSeedResult-csscWorkerValidFinalInc-run(\d)\.csv")
        for dir_ in os.listdir(root):
            dir_full = os.path.join(root, dir_)
            if not os.path.isdir(dir_full):
                continue
            scenario, solver, variant = dir_.split("_")
            solver = mapping_solver_name.get(solver, solver)
            track =  mapping_scenario_track[scenario]
            
            main_dict[track] = main_dict.get(track, {})
            track_dict = main_dict[track]
            track_dict[scenario] = track_dict.get(scenario, {})
            scenario_dict = track_dict[scenario]
            scenario_dict[solver] = scenario_dict.get(solver, {})
            solver_dict = scenario_dict[solver]
            
            dir_ = dir_full
            # attention: misleading file names
            def_train_file = os.path.join(dir_,"validationInstanceSeedResult-csscWorkerValidFinalInc-run0.csv")
            def_test_file = os.path.join(dir_, "validationInstanceSeedResult-csscWorkerValidFinalIncTest-run0.csv")
            
            # find configured results
            config_test_file = None
            config_train_file = None
            for element in os.listdir(dir_):
                if os.path.isdir(os.path.join(dir_,element)):
                    sub_dir = element
                    for element in os.listdir(os.path.join(dir_, sub_dir)):
                        match_config_test = config_test_regex.match(element)
                        if match_config_test:
                            config_test_file = os.path.join(dir_, sub_dir,element)
                        match_config_train = config_train_regex.match(element)
                        if match_config_train:
                            config_train_file = os.path.join(dir_, sub_dir,element)
                            fp = open(config_train_file, "r")
                            par1, par10, tos, perf_dict = self.read_csv(fp)
                            config_train_performance  = {"PAR1": par1, "PAR10": par10, "TOs": tos, "Performance": perf_dict}
                            fp.close() 
                            configurator = "%s-%s-%s" %(configurator_dic[sub_dir],match_config_train.group(1), variant)
                            solver_dict[configurator] =  {"training_perf" : config_train_performance}
                       
            if config_test_file is not None:
                # read data
                # default on train
                fp = open(def_train_file, "r")
                par1, par10, tos, perf_dict = self.read_csv(fp)
                solver_dict["default-train"]  = {"PAR1": par1, "PAR10": par10, "TOs": tos, "Performance": perf_dict}
                fp.close()   
                # default on test
                fp = open(def_test_file, "r")
                par1, par10, tos, perf_dict = self.read_csv(fp)
                solver_dict["default"]  = {"PAR1": par1, "PAR10": par10, "TOs": tos, "Performance": perf_dict}
                fp.close()
                # config on test
                fp = open(config_test_file, "r")
                par1, par10, tos, perf_dict = self.read_csv(fp)
                config_test_performance  = {"PAR1": par1, "PAR10": par10, "TOs": tos, "Performance": perf_dict}
                fp.close() 
                solver_dict["test_perf"] = config_test_performance
            else:
                logging.warn("Not found test performance for %s on %s" %(variant, scenario))
    
            best_configurator = self.get_best_train(solver_dict)
            solver_dict[best_configurator]["test_perf"] = solver_dict.get("test_perf", None)
        
        return main_dict
    
    
    def get_best_train(self, solver_dict):
        '''
            returns best configurator run of several independent runs
        '''
        best_perf = sys.maxint
        best_configurator = None
        for configurator, conf_dict in solver_dict.items():
            if configurator in ["default", "default-train", "test_perf"] :
                continue
            if best_perf > conf_dict["training_perf"]["PAR10"]:
                best_perf = conf_dict["training_perf"]["PAR10"]
                best_configurator = configurator
        return best_configurator
        
        
    def write_csv(self, main_dict):
        '''
            writes all results in csv files;
            naming convention: <track>_<scenario>_<solver>_<configurator>  
            columns: instance, default, configured
        '''
        # track 
        #       -> scenario 
        #         -> solver_dict 
        #           -> configurator/default/default-train
        #             ->  "training_perf"/"test_perf" 
        #               -> "PAR1"/"PAR10"/"TOs"/"Performance"
    
        logging.info("write CSV files")
        for track, track_dict in main_dict.items():
            for scen, scen_dict in track_dict.items():
                for solver, solver_dict in scen_dict.items():
                    for configurator, conf_dict in solver_dict.items():
                        #if "smac" not in configurator:
                        #    continue
                        if "default" in configurator or "test_perf" in configurator:
                            continue
                        
                        logging.debug("Write %s_%s_%s_%s.csv" %(track, scen, solver, configurator))
                        train_conf_perf = conf_dict["training_perf"]["Performance"]
                        train_default_perf = solver_dict["default-train"]["Performance"]
                        
                        test_default_perf = solver_dict["default"]["Performance"]
                        
                        try: 
                            configured_perf = conf_dict["test_perf"]["Performance"]
                            
                            insts = test_default_perf.keys()
                            
                            with open("%s_%s_%s_%s_TEST.csv" %(track, scen, solver, configurator),"w") as fp:
                                fp.write("instance,default_test,configured_test\n")
                                for i in insts:
                                    fp.write("%s,%f,%f\n" %(i, 
                                                                  max(0.005,test_default_perf[i]), 
                                                                  max(0.005,configured_perf[i])))

                        except KeyError:
                            logging.warn("Not found test perf (%s_%s_%s_%s)" %(track, scen, solver, configurator))
                        except TypeError:
                            logging.warn("Not found test perf (%s_%s_%s_%s)" %(track, scen, solver, configurator))
                        
                        insts = train_default_perf.keys()
                        logging.debug("Write %s_%s_%s_%s_TRAIN.csv" %(track, scen, solver, configurator))
                        with open("%s_%s_%s_%s_TRAIN.csv" %(track, scen, solver, configurator),"w") as fp:
                            fp.write("instance,default_train,configured_train\n")
                            for i in insts:
                                fp.write("%s,%f,%f\n" %(i, 
                                                        max(0.005, train_default_perf[i]),
                                                        max(0.005, train_conf_perf[i]),
                                                        ))
                    

if __name__ == '__main__':
    
    logging.basicConfig(level="DEBUG")
    
    if len(sys.argv) != 2:
        logging.error("Expected exactly one argument; path to root directory of log files")
        sys.exit(1)
    root = sys.argv[1]
    if not os.path.isdir(root):
        logging.error("Root Directory not found (%s)" %(root))
        sys.exit(2)
        
    rws = READ_AND_WRITE_CSV()
    rws.main(root)
    
    pass